// File edited 02/20/2015 with four space tab spacing

/** \file
	\brief Main file - this is where it all starts, and ends
*/

#include "simpletools.h"
#include "config.h"
//2/24/2015//Bruce//#include "serial.h"
#include "dda_queue.h"
#include "dda.h"
#include "gcode_parse.h"
//2/24/2015//Bruce//#include "timer.h"
//2/24/2015//Bruce//#include "temp.h"
#include "sermsg.h"
#include "debug.h"
#include "sersendf.h"
//2/24/2015//Bruce//#include "heater.h"
//2/24/2015//Bruce//#include "analog.h"
#include "pinio.h"
//2/24/2015//Bruce//#include "clock.h"

/// initialise all I/O - set pins as input or output, turn off unused subsystems, etc
void io_init(void)
{
	// setup I/O pins

	// X Stepper
	SET_DIRA_OUTPUT(X_STEP_PIN);
	SET_OUTA(X_STEP_PIN, 0);

	SET_DIRA_OUTPUT(X_DIR_PIN);
	SET_OUTA(X_DIR_PIN,  0);	

	#ifdef X_MIN_PIN
		SET_DIRA_INPUT(X_MIN_PIN);
		SET_OUTA(X_MIN_PIN, 0);
	#endif

	#ifdef X_MAX_PIN
		SET_DIRA_INPUT(X_MAX_PIN);
		SET_OUTA(X_MAX_PIN, 0);
	#endif

	// Y Stepper
	SET_DIRA_OUTPUT(Y_STEP_PIN);
	SET_OUTA(Y_STEP_PIN, 0);

	SET_DIRA_OUTPUT(Y_DIR_PIN);
	SET_OUTA(Y_DIR_PIN, 0);

	#ifdef Y_MIN_PIN
		SET_DIRA_INPUT(Y_MIN_PIN);
		SET_OUTA(Y_MIN_PIN, 0);
	#endif

	#ifdef Y_MAX_PIN
		SET_DIRA_INPUT(Y_MAX_PIN);
		SET_OUTA(Y_MAX_PIN, 0);
	#endif

	// Z Stepper
	SET_DIRA_OUTPUT(Z_STEP_PIN);
	SET_OUTA(Z_STEP_PIN, 0);
	
	SET_DIRA_OUTPUT(Z_DIR_PIN);
	SET_OUTA(Z_DIR_PIN, 0);	

	#ifdef Z_MIN_PIN
		SET_DIRA_INPUT(Z_MIN_PIN);
		SET_OUTA(Z_MIN_PIN, 0);
	#endif

	#ifdef Z_MAX_PIN
		SET_DIRA_INPUT(Z_MAX_PIN);
		SET_OUTA(Z_MAX_PIN, 0);
	#endif

	// E Stepper
	SET_DIRA_OUTPUT(E_STEP_PIN);
	SET_OUTA(E_STEP_PIN, 0);

	SET_DIRA_OUTPUT(E_DIR_PIN);
	SET_OUTA(E_DIR_PIN, 0);

	// Common Stepper Enable
	#ifdef STEPPER_ENABLE_PIN

		SET_DIRA_OUTPUT(STEPPER_ENABLE_PIN);

		#ifdef STEPPER_INVERT_ENABLE
			SET_OUTA(STEPPER_ENABLE_PIN, 0);
		#else
			SET_OUTA(STEPPER_ENABLE_PIN, 1);
		#endif
			
	#endif

	// X Stepper Enable
	#ifdef X_ENABLE_PIN

		SET_DIRA_OUTPUT(X_ENABLE_PIN);

		#ifdef X_INVERT_ENABLE
			SET_OUTA(X_ENABLE_PIN, 0);
		#else
			SET_OUTA(X_ENABLE_PIN, 1);
		#endif
		
	#endif

	// Y Stepper Enable
	#ifdef Y_ENABLE_PIN

		SET_DIRA_OUTPUT(Y_ENABLE_PIN);

		#ifdef Y_INVERT_ENABLE
			SET_OUTA(Y_ENABLE_PIN, 0);
		#else
			SET_OUTA(Y_ENABLE_PIN, 1);
		#endif
		
	#endif

	// Z Stepper Enable
	#ifdef Z_ENABLE_PIN

		SET_DIRA_OUTPUT(Z_ENABLE_PIN);

		#ifdef Z_INVERT_ENABLE
			SET_OUTA(Z_ENABLE_PIN, 0);
		#else
			SET_OUTA(Z_ENABLE_PIN, 1);
		#endif
		
	#endif

	// E Stepper Enable
	#ifdef E_ENABLE_PIN

		SET_DIRA_OUTPUT(E_ENABLE_PIN);

		#ifdef E_INVERT_ENABLE
			SET_OUTA(E_ENABLE_PIN, 0);
		#else
			SET_OUTA(E_ENABLE_PIN, 1);
		#endif
		
	#endif

	#ifdef	STEPPER_ENABLE_PIN
		power_off();
	#endif
}

/// Startup code, run when we come out of reset
void init(void)
{
	// set up serial
	//2/25/2015//serial_init();

	// set up G-code parsing
	gcode_init();

	// set up inputs and outputs
	io_init();

	// set up timers
	//2/25/2015//timer_init();

	// read PID settings from EEPROM
	//2/25/2015//heater_init();

	// set up dda
	dda_init();

	// start up analog read interrupt loop,
	// if any of the temp sensors in your config.h use analog interface
	//2/25/2015//analog_init();

	// enable interrupts
	//2/25/2015//sei();

	// say hi to host
	//2/25/2015//serial_writestr_P(PSTR("start\nok\n"));

}

/// this is where it all starts, and ends
///
/// just run init(), then run an endless loop where we pass characters from the serial RX buffer to gcode_parse_char() and check the clocks
int main (void)
{
	init();
 
	// main loop
	for (;;)
	{
		//2/25/2015//Bruce//
		/*
		// if queue is full, no point in reading chars- host will just have to wait
		if ((serial_rxchars() != 0) && (queue_full() == 0))
		{
			uint8_t c = serial_popchar();
			gcode_parse_char(c);
		}
		*/

		clock();
	}
}